<?php

/**
 * @file
 * Contains Drupal\cas_server\Ticket\TicketFactory
 */

namespace Drupal\cas_server\Ticket;

use Drupal\Component\Utility\Crypt;
use Drupal\cas_server\Configuration\ConfigHelper;
use Drupal\Core\Session\SessionManagerInterface;


class TicketFactory {

  /**
   * @var TicketStorageInterface
   *
   * The ticket store.
   */
  protected $ticketStore;

  /**
   * @var ConfigHelper
   *
   * The configuration helper.
   */
  protected $configHelper;

  /**
   * @var SessionManagerInterface
   *
   * The session manager.
   */
  protected $sessionManager;

  /**
   * Constructor.
   *
   * @param TicketStorageInterface $ticket_store
   *   The ticket store to use.
   * @param ConfigHelper $config_helper
   *   The configuration helper.
   */
  public function __construct(TicketStorageInterface $ticket_store, ConfigHelper $config_helper, SessionManagerInterface $session_manager) {
    $this->ticketStore = $ticket_store;
    $this->configHelper = $config_helper;
    $this->sessionManager = $session_manager;
  }

  /**
   * Create a proxy granting ticket.
   *
   * @param array $proxy_chain
   *   The proxy chain.
   *
   * @return ProxyGrantingTicket
   *   The created and saved PGT.
   */
  public function createProxyGrantingTicket($proxy_chain) {
    $id = 'PGT-';
    $id .= Crypt::randomBytesBase64(32);
    $expiration_time = REQUEST_TIME + $this->configHelper->getProxyGrantingTicketTimeout();
    $session = Crypt::hashBase64($this->sessionManager->getId());
    $name = \Drupal::currentUser()->getUsername();

    $pgt = new ProxyGrantingTicket($id, $expiration_time, $session, $name, $proxy_chain);
    $this->ticketStore->storeProxyGrantingTicket($pgt);

    return $pgt;

  }

  /**
   * Create a service ticket.
   *
   * @param string $service_string
   *   The service the ticket is being issued for.
   * @param bool $renew
   *   Whether or not this ticket was generated by directly supplied credentials.
   *
   * @return ServiceTicket
   *   The created and saved ServiceTicket.
   */
  public function createServiceTicket($service_string, $renew) {
    $id = 'ST-';
    $id .= Crypt::randomBytesBase64(32);
    $expiration_time = REQUEST_TIME + $this->configHelper->getServiceTicketTimeout();
    $session = Crypt::hashBase64($this->sessionManager->getId());
    $name = \Drupal::currentUser()->getUsername();

    $st = new ServiceTicket($id, $expiration_time, $session, $name, $service_string, $renew);
    $this->ticketStore->storeServiceTicket($st);

    return $st;
  }

  /**
   * Create a proxy ticket.
   *
   * @param string $service_string
   *   The service the ticket is being issued for.
   * @param bool $renew
   *   Whether or not this ticket was generated by directly supplied credentials.
   * @param array $proxy_chain
   *   The proxy chain.
   * @param string $session
   *   The session ID of the session who originally requested the requesting pgt.
   * @param string $name
   *   The username of the user who originally requested the requesting pgt.
   *
   * @return ProxyTicket
   *   The created and saved ProxyTicket.
   */
  public function createProxyTicket($service_string, $renew, $proxy_chain, $session, $name) {
    $id = 'PT-';
    $id .= Crypt::randomBytesBase64(32);
    $expiration_time = REQUEST_TIME + $this->configHelper->getProxyTicketTimeout();

    $pt = new ProxyTicket($id, $expiration_time, $session, $name, $service_string, $renew, $proxy_chain);
    $this->ticketStore->storeProxyTicket($pt);

    return $pt;
  }

  /**
   * Create a ticket granting ticket.
   *
   * @return TicketGrantingTicket
   *   The created and saved TicketGrantingTicket.
   */
  public function createTicketGrantingTicket() {
    $id = 'TGT-';
    $id .= Crypt::randomBytesBase64(32);
    $expiration_time = REQUEST_TIME + $this->configHelper->getTicketGrantingTicketTimeout();
    $session = Crypt::hashBase64($this->sessionManager->getId());
    $name = \Drupal::currentUser()->getUsername();

    $tgt = new TicketGrantingTicket($id, $expiration_time, $session, $name);
    $this->ticketStore->storeTicketGrantingTicket($tgt);

    return $tgt;
  }


}
